import 'dart:convert';
import 'package:flutter/material.dart';
import 'datamodel.dart';
import 'package:http/http.dart' as http;
import 'sensorconnector.dart';

class ViewEvents extends StatefulWidget {
  const ViewEvents({Key? key}) : super(key: key);

  @override
  _ViewEvents createState() => _ViewEvents();
}

class _ViewEvents extends State<ViewEvents> {
  // Get singleton to access locally stored events:
  EventStoreInstance events = EventStoreInstance();
  //TODO: add exception handling and display exceptions to user.

  Future<bool> syncEvents() async {
    //final EventStoreInstance events = EventStoreInstance();
    final ConfigurationStoreInstance configuration = ConfigurationStoreInstance();
    SensorConnector connection = SensorConnector();
    int _synccounter = 0;  //For displaying progress during event upload

    String? token = await connection.getAuthToken(configuration.loginInformation.mail,
        configuration.loginInformation.password);

    if (token != null) {
      _synccounter = events.getPendingEventCount();
      debugPrint('Number of Events: ' + events.events.length.toString());
      debugPrint('Number of Pending Events: ' + _synccounter.toString());
      var index = 0;
      for (var event in events.events) {
        if (event.status == 'PENDING') {
          debugPrint('Idx: ' + index.toString() + ' ' +
              event.toSensorJson().toString());
          index++;
          if (await connection.putEvent(event, token) == true){
            //Event has been posted to Sensor.
            _synccounter--;
            debugPrint('put success, remaining events: ' + _synccounter.toString());
            event.status = 'EXPORTED'; //Update event status so that it is only exported once.
            setState(() {});
          }
        }
      }
      return true;
    }
    return false;
  }

  @override
  Widget build(BuildContext context) {

    return Scaffold(
      appBar: AppBar(
        title: const Text("View Added Events"),
      ),
      body: Container(
      margin: const EdgeInsets.symmetric(horizontal: 5.0),
      child:
      SingleChildScrollView(
        scrollDirection: Axis.horizontal,
        child: SingleChildScrollView(
          scrollDirection: Axis.vertical,
          child: DataTable(

            columns: const <DataColumn>[
              DataColumn(
                label: Text(
                  'Id',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'URN',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Label',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Type',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Description',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'StartDate',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'EndDate',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Latitude',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Longitude',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Elevation',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
              DataColumn(
                label: Text(
                  'Status',
                  style: TextStyle(fontStyle: FontStyle.italic),
                ),
              ),
            ],
            rows: <DataRow>[
              for (var event in events.events)
                DataRow(
                  cells: <DataCell>[
                    DataCell(Text(event.id.toString())),
                    DataCell(Text(event.urn)),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        initialValue: event.label,
                        onFieldSubmitted: (val) {
                          event.label = val; //Update Database
                        },
                      ),
                    ),
                    DataCell(Text(event.type)),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        initialValue: event.description,
                        onFieldSubmitted: (val) {
                          event.description = val; //Update Database
                        },
                      ),
                    ),
                    DataCell(Text(event.startDate)),
                    DataCell(Text(event.endDate)),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        initialValue: event.latitude,
                        onFieldSubmitted: (val) {
                          event.latitude = val; //Update Database
                        },
                      ),
                    ),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        initialValue: event.longitude,
                        onFieldSubmitted: (val) {
                          event.longitude = val; //Update Database
                        },
                      ),
                    ),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        initialValue: event.elevation,
                        onFieldSubmitted: (val) {
                          event.elevation = val; //Update Database
                        },
                      ),
                    ),
                    DataCell(
                      TextFormField(
                        readOnly: true,
                        controller: TextEditingController(  //Required to update field here
                            text: event.status,
                        ),
                        onFieldSubmitted: (val) {
                          event.status = val; //Update Database
                        },
                      ),
                    ),
                  ],
                ),
            ],
          ),
        ),
      ),
      ),
      bottomNavigationBar: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text('Remaining events to sync: ' + events.getPendingEventCount().toString()),
          FloatingActionButton.extended(
            heroTag: null,
            tooltip: 'Upload Events',
            icon: null,
            label: const Text('Sync'),
            onPressed: () {

              syncEvents();
              //showDialog(
              //  barrierDismissible: false,
              //  context: context,
              //  builder: (BuildContext context) => _buildPopupDialog(context),
              //);
            },
          ),
        ],
      ),
    );
  }

  Widget _buildPopupDialog(BuildContext context) {

    return AlertDialog(
      title: const Text('Uploading Events'),

      content: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          //Text("Progress"),
          const CircularProgressIndicator(),
          const SizedBox(height: 10),
          Text('Remaining events to sync'),
        ],
      ),
      actions: <Widget>[
        TextButton(
          onPressed: () {
            //syncEvents();

            setState(() {

            });
            //Navigator.of(context).pop();
          },
          //textColor: Theme.of(context).primaryColor,
          child: const Text('Close'),
        ),
      ],
    );
  }
}

//TODO: allow editing fields here. Check validation of input value!